"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatNamedToolChoiceType = Literal["function",]


class ChatNamedToolChoiceFunctionTypedDict(TypedDict):
    name: str
    r"""Function name to call"""


class ChatNamedToolChoiceFunction(BaseModel):
    name: str
    r"""Function name to call"""


class ChatNamedToolChoiceTypedDict(TypedDict):
    r"""Named tool choice for specific function"""

    type: ChatNamedToolChoiceType
    function: ChatNamedToolChoiceFunctionTypedDict


class ChatNamedToolChoice(BaseModel):
    r"""Named tool choice for specific function"""

    type: ChatNamedToolChoiceType

    function: ChatNamedToolChoiceFunction



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List
from typing_extensions import Annotated, TypedDict


class ChatTokenLogprobTopLogprobTypedDict(TypedDict):
    token: str
    logprob: float
    bytes_: Nullable[List[float]]


class ChatTokenLogprobTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes_: Annotated[Nullable[List[float]], pydantic.Field(alias="bytes")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["bytes"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ChatTokenLogprobTypedDict(TypedDict):
    r"""Token log probability information"""

    token: str
    r"""The token"""
    logprob: float
    r"""Log probability of the token"""
    bytes_: Nullable[List[float]]
    r"""UTF-8 bytes of the token"""
    top_logprobs: List[ChatTokenLogprobTopLogprobTypedDict]
    r"""Top alternative tokens with probabilities"""


class ChatTokenLogprob(BaseModel):
    r"""Token log probability information"""

    token: str
    r"""The token"""

    logprob: float
    r"""Log probability of the token"""

    bytes_: Annotated[Nullable[List[float]], pydantic.Field(alias="bytes")]
    r"""UTF-8 bytes of the token"""

    top_logprobs: List[ChatTokenLogprobTopLogprob]
    r"""Top alternative tokens with probabilities"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["bytes"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChatStreamOptionsTypedDict(TypedDict):
    r"""Streaming configuration options"""

    include_usage: NotRequired[bool]
    r"""Deprecated: This field has no effect. Full usage details are always included."""


class ChatStreamOptions(BaseModel):
    r"""Streaming configuration options"""

    include_usage: Annotated[
        Optional[bool],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Deprecated: This field has no effect. Full usage details are always included."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesFunctionShellToolType = Literal["shell",]


class OpenResponsesFunctionShellToolTypedDict(TypedDict):
    r"""Shell tool configuration"""

    type: OpenResponsesFunctionShellToolType


class OpenResponsesFunctionShellTool(BaseModel):
    r"""Shell tool configuration"""

    type: OpenResponsesFunctionShellToolType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .jsonschemaconfig import JSONSchemaConfig, JSONSchemaConfigTypedDict
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponseFormatJSONSchemaType = Literal["json_schema",]


class ResponseFormatJSONSchemaTypedDict(TypedDict):
    r"""JSON Schema response format for structured outputs"""

    type: ResponseFormatJSONSchemaType
    json_schema: JSONSchemaConfigTypedDict
    r"""JSON Schema configuration object"""


class ResponseFormatJSONSchema(BaseModel):
    r"""JSON Schema response format for structured outputs"""

    type: ResponseFormatJSONSchemaType

    json_schema: JSONSchemaConfig
    r"""JSON Schema configuration object"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


FunctionCallArgsDeltaEventType = Literal["response.function_call_arguments.delta",]


class FunctionCallArgsDeltaEventTypedDict(TypedDict):
    r"""Event emitted when function call arguments are being streamed"""

    type: FunctionCallArgsDeltaEventType
    item_id: str
    output_index: float
    delta: str
    sequence_number: float


class FunctionCallArgsDeltaEvent(BaseModel):
    r"""Event emitted when function call arguments are being streamed"""

    type: FunctionCallArgsDeltaEventType

    item_id: str

    output_index: float

    delta: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ResponseInputFileType = Literal["input_file",]


class ResponseInputFileTypedDict(TypedDict):
    r"""File input content item"""

    type: ResponseInputFileType
    file_id: NotRequired[Nullable[str]]
    file_data: NotRequired[str]
    filename: NotRequired[str]
    file_url: NotRequired[str]


class ResponseInputFile(BaseModel):
    r"""File input content item"""

    type: ResponseInputFileType

    file_id: OptionalNullable[str] = UNSET

    file_data: Optional[str] = None

    filename: Optional[str] = None

    file_url: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["file_id", "file_data", "filename", "file_url"]
        nullable_fields = ["file_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import (
    chatrequest as components_chatrequest,
    chatresult as components_chatresult,
    chatstreamchunk as components_chatstreamchunk,
)
from openrouter.types import BaseModel
from openrouter.utils import (
    FieldMetadata,
    HeaderMetadata,
    RequestMetadata,
    eventstreaming,
)
import pydantic
from typing import Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class SendChatCompletionRequestGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class SendChatCompletionRequestGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class SendChatCompletionRequestRequestTypedDict(TypedDict):
    chat_request: components_chatrequest.ChatRequestTypedDict
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class SendChatCompletionRequestRequest(BaseModel):
    chat_request: Annotated[
        components_chatrequest.ChatRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class SendChatCompletionRequestResponseBodyTypedDict(TypedDict):
    r"""Successful chat completion response"""

    data: components_chatstreamchunk.ChatStreamChunkTypedDict
    r"""Streaming chat completion chunk"""


class SendChatCompletionRequestResponseBody(BaseModel):
    r"""Successful chat completion response"""

    data: components_chatstreamchunk.ChatStreamChunk
    r"""Streaming chat completion chunk"""


SendChatCompletionRequestResponseTypedDict = TypeAliasType(
    "SendChatCompletionRequestResponseTypedDict",
    Union[
        components_chatresult.ChatResultTypedDict,
        Union[
            eventstreaming.EventStream[
                components_chatstreamchunk.ChatStreamChunkTypedDict
            ],
            eventstreaming.EventStreamAsync[
                components_chatstreamchunk.ChatStreamChunkTypedDict
            ],
        ],
    ],
)


SendChatCompletionRequestResponse = TypeAliasType(
    "SendChatCompletionRequestResponse",
    Union[
        components_chatresult.ChatResult,
        Union[
            eventstreaming.EventStream[components_chatstreamchunk.ChatStreamChunk],
            eventstreaming.EventStreamAsync[components_chatstreamchunk.ChatStreamChunk],
        ],
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .bulkassignkeystoguardrail import (
        BulkAssignKeysToGuardrailGlobals,
        BulkAssignKeysToGuardrailGlobalsTypedDict,
        BulkAssignKeysToGuardrailRequest,
        BulkAssignKeysToGuardrailRequestBody,
        BulkAssignKeysToGuardrailRequestBodyTypedDict,
        BulkAssignKeysToGuardrailRequestTypedDict,
        BulkAssignKeysToGuardrailResponse,
        BulkAssignKeysToGuardrailResponseTypedDict,
    )
    from .bulkassignmemberstoguardrail import (
        BulkAssignMembersToGuardrailGlobals,
        BulkAssignMembersToGuardrailGlobalsTypedDict,
        BulkAssignMembersToGuardrailRequest,
        BulkAssignMembersToGuardrailRequestBody,
        BulkAssignMembersToGuardrailRequestBodyTypedDict,
        BulkAssignMembersToGuardrailRequestTypedDict,
        BulkAssignMembersToGuardrailResponse,
        BulkAssignMembersToGuardrailResponseTypedDict,
    )
    from .bulkunassignkeysfromguardrail import (
        BulkUnassignKeysFromGuardrailGlobals,
        BulkUnassignKeysFromGuardrailGlobalsTypedDict,
        BulkUnassignKeysFromGuardrailRequest,
        BulkUnassignKeysFromGuardrailRequestBody,
        BulkUnassignKeysFromGuardrailRequestBodyTypedDict,
        BulkUnassignKeysFromGuardrailRequestTypedDict,
        BulkUnassignKeysFromGuardrailResponse,
        BulkUnassignKeysFromGuardrailResponseTypedDict,
    )
    from .bulkunassignmembersfromguardrail import (
        BulkUnassignMembersFromGuardrailGlobals,
        BulkUnassignMembersFromGuardrailGlobalsTypedDict,
        BulkUnassignMembersFromGuardrailRequest,
        BulkUnassignMembersFromGuardrailRequestBody,
        BulkUnassignMembersFromGuardrailRequestBodyTypedDict,
        BulkUnassignMembersFromGuardrailRequestTypedDict,
        BulkUnassignMembersFromGuardrailResponse,
        BulkUnassignMembersFromGuardrailResponseTypedDict,
    )
    from .createauthkeyscode import (
        CreateAuthKeysCodeCodeChallengeMethod,
        CreateAuthKeysCodeData,
        CreateAuthKeysCodeDataTypedDict,
        CreateAuthKeysCodeGlobals,
        CreateAuthKeysCodeGlobalsTypedDict,
        CreateAuthKeysCodeRequest,
        CreateAuthKeysCodeRequestBody,
        CreateAuthKeysCodeRequestBodyTypedDict,
        CreateAuthKeysCodeRequestTypedDict,
        CreateAuthKeysCodeResponse,
        CreateAuthKeysCodeResponseTypedDict,
        UsageLimitType,
    )
    from .createcoinbasecharge import (
        CallData,
        CallDataTypedDict,
        CreateCoinbaseChargeData,
        CreateCoinbaseChargeDataTypedDict,
        CreateCoinbaseChargeGlobals,
        CreateCoinbaseChargeGlobalsTypedDict,
        CreateCoinbaseChargeRequest,
        CreateCoinbaseChargeRequestTypedDict,
        CreateCoinbaseChargeResponse,
        CreateCoinbaseChargeResponseTypedDict,
        CreateCoinbaseChargeSecurity,
        CreateCoinbaseChargeSecurityTypedDict,
        Metadata,
        MetadataTypedDict,
        TransferIntent,
        TransferIntentTypedDict,
        Web3Data,
        Web3DataTypedDict,
    )
    from .createembeddings import (
        Content,
        ContentImageURL,
        ContentImageURLTypedDict,
        ContentText,
        ContentTextTypedDict,
        ContentTypedDict,
        CreateEmbeddingsData,
        CreateEmbeddingsDataTypedDict,
        CreateEmbeddingsGlobals,
        CreateEmbeddingsGlobalsTypedDict,
        CreateEmbeddingsRequest,
        CreateEmbeddingsRequestBody,
        CreateEmbeddingsRequestBodyTypedDict,
        CreateEmbeddingsRequestTypedDict,
        CreateEmbeddingsResponse,
        CreateEmbeddingsResponseBody,
        CreateEmbeddingsResponseBodyTypedDict,
        CreateEmbeddingsResponseTypedDict,
        Embedding,
        EmbeddingTypedDict,
        EncodingFormat,
        ImageURL,
        ImageURLTypedDict,
        Input,
        InputTypedDict,
        InputUnion,
        InputUnionTypedDict,
        Object,
        ObjectEmbedding,
        TypeImageURL,
        TypeText,
        Usage,
        UsageTypedDict,
    )
    from .createguardrail import (
        CreateGuardrailData,
        CreateGuardrailDataTypedDict,
        CreateGuardrailGlobals,
        CreateGuardrailGlobalsTypedDict,
        CreateGuardrailRequest,
        CreateGuardrailRequestBody,
        CreateGuardrailRequestBodyTypedDict,
        CreateGuardrailRequestTypedDict,
        CreateGuardrailResetIntervalRequest,
        CreateGuardrailResetIntervalResponse,
        CreateGuardrailResponse,
        CreateGuardrailResponseTypedDict,
    )
    from .createkeys import (
        CreateKeysData,
        CreateKeysDataTypedDict,
        CreateKeysGlobals,
        CreateKeysGlobalsTypedDict,
        CreateKeysLimitReset,
        CreateKeysRequest,
        CreateKeysRequestBody,
        CreateKeysRequestBodyTypedDict,
        CreateKeysRequestTypedDict,
        CreateKeysResponse,
        CreateKeysResponseTypedDict,
    )
    from .createresponses import (
        CreateResponsesGlobals,
        CreateResponsesGlobalsTypedDict,
        CreateResponsesRequest,
        CreateResponsesRequestTypedDict,
        CreateResponsesResponse,
        CreateResponsesResponseBody,
        CreateResponsesResponseBodyTypedDict,
        CreateResponsesResponseTypedDict,
    )
    from .deleteguardrail import (
        DeleteGuardrailGlobals,
        DeleteGuardrailGlobalsTypedDict,
        DeleteGuardrailRequest,
        DeleteGuardrailRequestTypedDict,
        DeleteGuardrailResponse,
        DeleteGuardrailResponseTypedDict,
    )
    from .deletekeys import (
        DeleteKeysGlobals,
        DeleteKeysGlobalsTypedDict,
        DeleteKeysRequest,
        DeleteKeysRequestTypedDict,
        DeleteKeysResponse,
        DeleteKeysResponseTypedDict,
    )
    from .exchangeauthcodeforapikey import (
        ExchangeAuthCodeForAPIKeyCodeChallengeMethod,
        ExchangeAuthCodeForAPIKeyGlobals,
        ExchangeAuthCodeForAPIKeyGlobalsTypedDict,
        ExchangeAuthCodeForAPIKeyRequest,
        ExchangeAuthCodeForAPIKeyRequestBody,
        ExchangeAuthCodeForAPIKeyRequestBodyTypedDict,
        ExchangeAuthCodeForAPIKeyRequestTypedDict,
        ExchangeAuthCodeForAPIKeyResponse,
        ExchangeAuthCodeForAPIKeyResponseTypedDict,
    )
    from .getcredits import (
        GetCreditsData,
        GetCreditsDataTypedDict,
        GetCreditsGlobals,
        GetCreditsGlobalsTypedDict,
        GetCreditsRequest,
        GetCreditsRequestTypedDict,
        GetCreditsResponse,
        GetCreditsResponseTypedDict,
    )
    from .getcurrentkey import (
        GetCurrentKeyData,
        GetCurrentKeyDataTypedDict,
        GetCurrentKeyGlobals,
        GetCurrentKeyGlobalsTypedDict,
        GetCurrentKeyRequest,
        GetCurrentKeyRequestTypedDict,
        GetCurrentKeyResponse,
        GetCurrentKeyResponseTypedDict,
        RateLimit,
        RateLimitTypedDict,
    )
    from .getgeneration import (
        APIType,
        GetGenerationData,
        GetGenerationDataTypedDict,
        GetGenerationGlobals,
        GetGenerationGlobalsTypedDict,
        GetGenerationRequest,
        GetGenerationRequestTypedDict,
        GetGenerationResponse,
        GetGenerationResponseTypedDict,
        ProviderName,
        ProviderResponse,
        ProviderResponseTypedDict,
    )
    from .getguardrail import (
        GetGuardrailData,
        GetGuardrailDataTypedDict,
        GetGuardrailGlobals,
        GetGuardrailGlobalsTypedDict,
        GetGuardrailRequest,
        GetGuardrailRequestTypedDict,
        GetGuardrailResetInterval,
        GetGuardrailResponse,
        GetGuardrailResponseTypedDict,
    )
    from .getkey import (
        GetKeyData,
        GetKeyDataTypedDict,
        GetKeyGlobals,
        GetKeyGlobalsTypedDict,
        GetKeyRequest,
        GetKeyRequestTypedDict,
        GetKeyResponse,
        GetKeyResponseTypedDict,
    )
    from .getmodels import (
        Category,
        GetModelsGlobals,
        GetModelsGlobalsTypedDict,
        GetModelsRequest,
        GetModelsRequestTypedDict,
    )
    from .getuseractivity import (
        GetUserActivityGlobals,
        GetUserActivityGlobalsTypedDict,
        GetUserActivityRequest,
        GetUserActivityRequestTypedDict,
        GetUserActivityResponse,
        GetUserActivityResponseTypedDict,
    )
    from .list import (
        ListData,
        ListDataTypedDict,
        ListGlobals,
        ListGlobalsTypedDict,
        ListRequest,
        ListRequestTypedDict,
        ListResponse,
        ListResponseTypedDict,
    )
    from .listembeddingsmodels import (
        ListEmbeddingsModelsGlobals,
        ListEmbeddingsModelsGlobalsTypedDict,
        ListEmbeddingsModelsRequest,
        ListEmbeddingsModelsRequestTypedDict,
    )
    from .listendpoints import (
        ListEndpointsGlobals,
        ListEndpointsGlobalsTypedDict,
        ListEndpointsRequest,
        ListEndpointsRequestTypedDict,
        ListEndpointsResponse,
        ListEndpointsResponseTypedDict,
    )
    from .listendpointszdr import (
        ListEndpointsZdrGlobals,
        ListEndpointsZdrGlobalsTypedDict,
        ListEndpointsZdrRequest,
        ListEndpointsZdrRequestTypedDict,
        ListEndpointsZdrResponse,
        ListEndpointsZdrResponseTypedDict,
    )
    from .listguardrailkeyassignments import (
        ListGuardrailKeyAssignmentsData,
        ListGuardrailKeyAssignmentsDataTypedDict,
        ListGuardrailKeyAssignmentsGlobals,
        ListGuardrailKeyAssignmentsGlobalsTypedDict,
        ListGuardrailKeyAssignmentsRequest,
        ListGuardrailKeyAssignmentsRequestTypedDict,
        ListGuardrailKeyAssignmentsResponse,
        ListGuardrailKeyAssignmentsResponseTypedDict,
    )
    from .listguardrailmemberassignments import (
        ListGuardrailMemberAssignmentsData,
        ListGuardrailMemberAssignmentsDataTypedDict,
        ListGuardrailMemberAssignmentsGlobals,
        ListGuardrailMemberAssignmentsGlobalsTypedDict,
        ListGuardrailMemberAssignmentsRequest,
        ListGuardrailMemberAssignmentsRequestTypedDict,
        ListGuardrailMemberAssignmentsResponse,
        ListGuardrailMemberAssignmentsResponseTypedDict,
    )
    from .listguardrails import (
        ListGuardrailsData,
        ListGuardrailsDataTypedDict,
        ListGuardrailsGlobals,
        ListGuardrailsGlobalsTypedDict,
        ListGuardrailsRequest,
        ListGuardrailsRequestTypedDict,
        ListGuardrailsResetInterval,
        ListGuardrailsResponse,
        ListGuardrailsResponseTypedDict,
    )
    from .listkeyassignments import (
        ListKeyAssignmentsData,
        ListKeyAssignmentsDataTypedDict,
        ListKeyAssignmentsGlobals,
        ListKeyAssignmentsGlobalsTypedDict,
        ListKeyAssignmentsRequest,
        ListKeyAssignmentsRequestTypedDict,
        ListKeyAssignmentsResponse,
        ListKeyAssignmentsResponseTypedDict,
    )
    from .listmemberassignments import (
        ListMemberAssignmentsData,
        ListMemberAssignmentsDataTypedDict,
        ListMemberAssignmentsGlobals,
        ListMemberAssignmentsGlobalsTypedDict,
        ListMemberAssignmentsRequest,
        ListMemberAssignmentsRequestTypedDict,
        ListMemberAssignmentsResponse,
        ListMemberAssignmentsResponseTypedDict,
    )
    from .listmodelscount import (
        ListModelsCountGlobals,
        ListModelsCountGlobalsTypedDict,
        ListModelsCountRequest,
        ListModelsCountRequestTypedDict,
    )
    from .listmodelsuser import (
        ListModelsUserGlobals,
        ListModelsUserGlobalsTypedDict,
        ListModelsUserRequest,
        ListModelsUserRequestTypedDict,
        ListModelsUserSecurity,
        ListModelsUserSecurityTypedDict,
    )
    from .listproviders import (
        Datacenter,
        Headquarters,
        ListProvidersData,
        ListProvidersDataTypedDict,
        ListProvidersGlobals,
        ListProvidersGlobalsTypedDict,
        ListProvidersRequest,
        ListProvidersRequestTypedDict,
        ListProvidersResponse,
        ListProvidersResponseTypedDict,
    )
    from .sendchatcompletionrequest import (
        SendChatCompletionRequestGlobals,
        SendChatCompletionRequestGlobalsTypedDict,
        SendChatCompletionRequestRequest,
        SendChatCompletionRequestRequestTypedDict,
        SendChatCompletionRequestResponse,
        SendChatCompletionRequestResponseBody,
        SendChatCompletionRequestResponseBodyTypedDict,
        SendChatCompletionRequestResponseTypedDict,
    )
    from .updateguardrail import (
        UpdateGuardrailData,
        UpdateGuardrailDataTypedDict,
        UpdateGuardrailGlobals,
        UpdateGuardrailGlobalsTypedDict,
        UpdateGuardrailRequest,
        UpdateGuardrailRequestBody,
        UpdateGuardrailRequestBodyTypedDict,
        UpdateGuardrailRequestTypedDict,
        UpdateGuardrailResetIntervalRequest,
        UpdateGuardrailResetIntervalResponse,
        UpdateGuardrailResponse,
        UpdateGuardrailResponseTypedDict,
    )
    from .updatekeys import (
        UpdateKeysData,
        UpdateKeysDataTypedDict,
        UpdateKeysGlobals,
        UpdateKeysGlobalsTypedDict,
        UpdateKeysLimitReset,
        UpdateKeysRequest,
        UpdateKeysRequestBody,
        UpdateKeysRequestBodyTypedDict,
        UpdateKeysRequestTypedDict,
        UpdateKeysResponse,
        UpdateKeysResponseTypedDict,
    )

__all__ = [
    "APIType",
    "BulkAssignKeysToGuardrailGlobals",
    "BulkAssignKeysToGuardrailGlobalsTypedDict",
    "BulkAssignKeysToGuardrailRequest",
    "BulkAssignKeysToGuardrailRequestBody",
    "BulkAssignKeysToGuardrailRequestBodyTypedDict",
    "BulkAssignKeysToGuardrailRequestTypedDict",
    "BulkAssignKeysToGuardrailResponse",
    "BulkAssignKeysToGuardrailResponseTypedDict",
    "BulkAssignMembersToGuardrailGlobals",
    "BulkAssignMembersToGuardrailGlobalsTypedDict",
    "BulkAssignMembersToGuardrailRequest",
    "BulkAssignMembersToGuardrailRequestBody",
    "BulkAssignMembersToGuardrailRequestBodyTypedDict",
    "BulkAssignMembersToGuardrailRequestTypedDict",
    "BulkAssignMembersToGuardrailResponse",
    "BulkAssignMembersToGuardrailResponseTypedDict",
    "BulkUnassignKeysFromGuardrailGlobals",
    "BulkUnassignKeysFromGuardrailGlobalsTypedDict",
    "BulkUnassignKeysFromGuardrailRequest",
    "BulkUnassignKeysFromGuardrailRequestBody",
    "BulkUnassignKeysFromGuardrailRequestBodyTypedDict",
    "BulkUnassignKeysFromGuardrailRequestTypedDict",
    "BulkUnassignKeysFromGuardrailResponse",
    "BulkUnassignKeysFromGuardrailResponseTypedDict",
    "BulkUnassignMembersFromGuardrailGlobals",
    "BulkUnassignMembersFromGuardrailGlobalsTypedDict",
    "BulkUnassignMembersFromGuardrailRequest",
    "BulkUnassignMembersFromGuardrailRequestBody",
    "BulkUnassignMembersFromGuardrailRequestBodyTypedDict",
    "BulkUnassignMembersFromGuardrailRequestTypedDict",
    "BulkUnassignMembersFromGuardrailResponse",
    "BulkUnassignMembersFromGuardrailResponseTypedDict",
    "CallData",
    "CallDataTypedDict",
    "Category",
    "Content",
    "ContentImageURL",
    "ContentImageURLTypedDict",
    "ContentText",
    "ContentTextTypedDict",
    "ContentTypedDict",
    "CreateAuthKeysCodeCodeChallengeMethod",
    "CreateAuthKeysCodeData",
    "CreateAuthKeysCodeDataTypedDict",
    "CreateAuthKeysCodeGlobals",
    "CreateAuthKeysCodeGlobalsTypedDict",
    "CreateAuthKeysCodeRequest",
    "CreateAuthKeysCodeRequestBody",
    "CreateAuthKeysCodeRequestBodyTypedDict",
    "CreateAuthKeysCodeRequestTypedDict",
    "CreateAuthKeysCodeResponse",
    "CreateAuthKeysCodeResponseTypedDict",
    "CreateCoinbaseChargeData",
    "CreateCoinbaseChargeDataTypedDict",
    "CreateCoinbaseChargeGlobals",
    "CreateCoinbaseChargeGlobalsTypedDict",
    "CreateCoinbaseChargeRequest",
    "CreateCoinbaseChargeRequestTypedDict",
    "CreateCoinbaseChargeResponse",
    "CreateCoinbaseChargeResponseTypedDict",
    "CreateCoinbaseChargeSecurity",
    "CreateCoinbaseChargeSecurityTypedDict",
    "CreateEmbeddingsData",
    "CreateEmbeddingsDataTypedDict",
    "CreateEmbeddingsGlobals",
    "CreateEmbeddingsGlobalsTypedDict",
    "CreateEmbeddingsRequest",
    "CreateEmbeddingsRequestBody",
    "CreateEmbeddingsRequestBodyTypedDict",
    "CreateEmbeddingsRequestTypedDict",
    "CreateEmbeddingsResponse",
    "CreateEmbeddingsResponseBody",
    "CreateEmbeddingsResponseBodyTypedDict",
    "CreateEmbeddingsResponseTypedDict",
    "CreateGuardrailData",
    "CreateGuardrailDataTypedDict",
    "CreateGuardrailGlobals",
    "CreateGuardrailGlobalsTypedDict",
    "CreateGuardrailRequest",
    "CreateGuardrailRequestBody",
    "CreateGuardrailRequestBodyTypedDict",
    "CreateGuardrailRequestTypedDict",
    "CreateGuardrailResetIntervalRequest",
    "CreateGuardrailResetIntervalResponse",
    "CreateGuardrailResponse",
    "CreateGuardrailResponseTypedDict",
    "CreateKeysData",
    "CreateKeysDataTypedDict",
    "CreateKeysGlobals",
    "CreateKeysGlobalsTypedDict",
    "CreateKeysLimitReset",
    "CreateKeysRequest",
    "CreateKeysRequestBody",
    "CreateKeysRequestBodyTypedDict",
    "CreateKeysRequestTypedDict",
    "CreateKeysResponse",
    "CreateKeysResponseTypedDict",
    "CreateResponsesGlobals",
    "CreateResponsesGlobalsTypedDict",
    "CreateResponsesRequest",
    "CreateResponsesRequestTypedDict",
    "CreateResponsesResponse",
    "CreateResponsesResponseBody",
    "CreateResponsesResponseBodyTypedDict",
    "CreateResponsesResponseTypedDict",
    "Datacenter",
    "DeleteGuardrailGlobals",
    "DeleteGuardrailGlobalsTypedDict",
    "DeleteGuardrailRequest",
    "DeleteGuardrailRequestTypedDict",
    "DeleteGuardrailResponse",
    "DeleteGuardrailResponseTypedDict",
    "DeleteKeysGlobals",
    "DeleteKeysGlobalsTypedDict",
    "DeleteKeysRequest",
    "DeleteKeysRequestTypedDict",
    "DeleteKeysResponse",
    "DeleteKeysResponseTypedDict",
    "Embedding",
    "EmbeddingTypedDict",
    "EncodingFormat",
    "ExchangeAuthCodeForAPIKeyCodeChallengeMethod",
    "ExchangeAuthCodeForAPIKeyGlobals",
    "ExchangeAuthCodeForAPIKeyGlobalsTypedDict",
    "ExchangeAuthCodeForAPIKeyRequest",
    "ExchangeAuthCodeForAPIKeyRequestBody",
    "ExchangeAuthCodeForAPIKeyRequestBodyTypedDict",
    "ExchangeAuthCodeForAPIKeyRequestTypedDict",
    "ExchangeAuthCodeForAPIKeyResponse",
    "ExchangeAuthCodeForAPIKeyResponseTypedDict",
    "GetCreditsData",
    "GetCreditsDataTypedDict",
    "GetCreditsGlobals",
    "GetCreditsGlobalsTypedDict",
    "GetCreditsRequest",
    "GetCreditsRequestTypedDict",
    "GetCreditsResponse",
    "GetCreditsResponseTypedDict",
    "GetCurrentKeyData",
    "GetCurrentKeyDataTypedDict",
    "GetCurrentKeyGlobals",
    "GetCurrentKeyGlobalsTypedDict",
    "GetCurrentKeyRequest",
    "GetCurrentKeyRequestTypedDict",
    "GetCurrentKeyResponse",
    "GetCurrentKeyResponseTypedDict",
    "GetGenerationData",
    "GetGenerationDataTypedDict",
    "GetGenerationGlobals",
    "GetGenerationGlobalsTypedDict",
    "GetGenerationRequest",
    "GetGenerationRequestTypedDict",
    "GetGenerationResponse",
    "GetGenerationResponseTypedDict",
    "GetGuardrailData",
    "GetGuardrailDataTypedDict",
    "GetGuardrailGlobals",
    "GetGuardrailGlobalsTypedDict",
    "GetGuardrailRequest",
    "GetGuardrailRequestTypedDict",
    "GetGuardrailResetInterval",
    "GetGuardrailResponse",
    "GetGuardrailResponseTypedDict",
    "GetKeyData",
    "GetKeyDataTypedDict",
    "GetKeyGlobals",
    "GetKeyGlobalsTypedDict",
    "GetKeyRequest",
    "GetKeyRequestTypedDict",
    "GetKeyResponse",
    "GetKeyResponseTypedDict",
    "GetModelsGlobals",
    "GetModelsGlobalsTypedDict",
    "GetModelsRequest",
    "GetModelsRequestTypedDict",
    "GetUserActivityGlobals",
    "GetUserActivityGlobalsTypedDict",
    "GetUserActivityRequest",
    "GetUserActivityRequestTypedDict",
    "GetUserActivityResponse",
    "GetUserActivityResponseTypedDict",
    "Headquarters",
    "ImageURL",
    "ImageURLTypedDict",
    "Input",
    "InputTypedDict",
    "InputUnion",
    "InputUnionTypedDict",
    "ListData",
    "ListDataTypedDict",
    "ListEmbeddingsModelsGlobals",
    "ListEmbeddingsModelsGlobalsTypedDict",
    "ListEmbeddingsModelsRequest",
    "ListEmbeddingsModelsRequestTypedDict",
    "ListEndpointsGlobals",
    "ListEndpointsGlobalsTypedDict",
    "ListEndpointsRequest",
    "ListEndpointsRequestTypedDict",
    "ListEndpointsResponse",
    "ListEndpointsResponseTypedDict",
    "ListEndpointsZdrGlobals",
    "ListEndpointsZdrGlobalsTypedDict",
    "ListEndpointsZdrRequest",
    "ListEndpointsZdrRequestTypedDict",
    "ListEndpointsZdrResponse",
    "ListEndpointsZdrResponseTypedDict",
    "ListGlobals",
    "ListGlobalsTypedDict",
    "ListGuardrailKeyAssignmentsData",
    "ListGuardrailKeyAssignmentsDataTypedDict",
    "ListGuardrailKeyAssignmentsGlobals",
    "ListGuardrailKeyAssignmentsGlobalsTypedDict",
    "ListGuardrailKeyAssignmentsRequest",
    "ListGuardrailKeyAssignmentsRequestTypedDict",
    "ListGuardrailKeyAssignmentsResponse",
    "ListGuardrailKeyAssignmentsResponseTypedDict",
    "ListGuardrailMemberAssignmentsData",
    "ListGuardrailMemberAssignmentsDataTypedDict",
    "ListGuardrailMemberAssignmentsGlobals",
    "ListGuardrailMemberAssignmentsGlobalsTypedDict",
    "ListGuardrailMemberAssignmentsRequest",
    "ListGuardrailMemberAssignmentsRequestTypedDict",
    "ListGuardrailMemberAssignmentsResponse",
    "ListGuardrailMemberAssignmentsResponseTypedDict",
    "ListGuardrailsData",
    "ListGuardrailsDataTypedDict",
    "ListGuardrailsGlobals",
    "ListGuardrailsGlobalsTypedDict",
    "ListGuardrailsRequest",
    "ListGuardrailsRequestTypedDict",
    "ListGuardrailsResetInterval",
    "ListGuardrailsResponse",
    "ListGuardrailsResponseTypedDict",
    "ListKeyAssignmentsData",
    "ListKeyAssignmentsDataTypedDict",
    "ListKeyAssignmentsGlobals",
    "ListKeyAssignmentsGlobalsTypedDict",
    "ListKeyAssignmentsRequest",
    "ListKeyAssignmentsRequestTypedDict",
    "ListKeyAssignmentsResponse",
    "ListKeyAssignmentsResponseTypedDict",
    "ListMemberAssignmentsData",
    "ListMemberAssignmentsDataTypedDict",
    "ListMemberAssignmentsGlobals",
    "ListMemberAssignmentsGlobalsTypedDict",
    "ListMemberAssignmentsRequest",
    "ListMemberAssignmentsRequestTypedDict",
    "ListMemberAssignmentsResponse",
    "ListMemberAssignmentsResponseTypedDict",
    "ListModelsCountGlobals",
    "ListModelsCountGlobalsTypedDict",
    "ListModelsCountRequest",
    "ListModelsCountRequestTypedDict",
    "ListModelsUserGlobals",
    "ListModelsUserGlobalsTypedDict",
    "ListModelsUserRequest",
    "ListModelsUserRequestTypedDict",
    "ListModelsUserSecurity",
    "ListModelsUserSecurityTypedDict",
    "ListProvidersData",
    "ListProvidersDataTypedDict",
    "ListProvidersGlobals",
    "ListProvidersGlobalsTypedDict",
    "ListProvidersRequest",
    "ListProvidersRequestTypedDict",
    "ListProvidersResponse",
    "ListProvidersResponseTypedDict",
    "ListRequest",
    "ListRequestTypedDict",
    "ListResponse",
    "ListResponseTypedDict",
    "Metadata",
    "MetadataTypedDict",
    "Object",
    "ObjectEmbedding",
    "ProviderName",
    "ProviderResponse",
    "ProviderResponseTypedDict",
    "RateLimit",
    "RateLimitTypedDict",
    "SendChatCompletionRequestGlobals",
    "SendChatCompletionRequestGlobalsTypedDict",
    "SendChatCompletionRequestRequest",
    "SendChatCompletionRequestRequestTypedDict",
    "SendChatCompletionRequestResponse",
    "SendChatCompletionRequestResponseBody",
    "SendChatCompletionRequestResponseBodyTypedDict",
    "SendChatCompletionRequestResponseTypedDict",
    "TransferIntent",
    "TransferIntentTypedDict",
    "TypeImageURL",
    "TypeText",
    "UpdateGuardrailData",
    "UpdateGuardrailDataTypedDict",
    "UpdateGuardrailGlobals",
    "UpdateGuardrailGlobalsTypedDict",
    "UpdateGuardrailRequest",
    "UpdateGuardrailRequestBody",
    "UpdateGuardrailRequestBodyTypedDict",
    "UpdateGuardrailRequestTypedDict",
    "UpdateGuardrailResetIntervalRequest",
    "UpdateGuardrailResetIntervalResponse",
    "UpdateGuardrailResponse",
    "UpdateGuardrailResponseTypedDict",
    "UpdateKeysData",
    "UpdateKeysDataTypedDict",
    "UpdateKeysGlobals",
    "UpdateKeysGlobalsTypedDict",
    "UpdateKeysLimitReset",
    "UpdateKeysRequest",
    "UpdateKeysRequestBody",
    "UpdateKeysRequestBodyTypedDict",
    "UpdateKeysRequestTypedDict",
    "UpdateKeysResponse",
    "UpdateKeysResponseTypedDict",
    "Usage",
    "UsageLimitType",
    "UsageTypedDict",
    "Web3Data",
    "Web3DataTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "BulkAssignKeysToGuardrailGlobals": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailGlobalsTypedDict": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailRequest": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailRequestBody": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailRequestBodyTypedDict": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailRequestTypedDict": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailResponse": ".bulkassignkeystoguardrail",
    "BulkAssignKeysToGuardrailResponseTypedDict": ".bulkassignkeystoguardrail",
    "BulkAssignMembersToGuardrailGlobals": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailGlobalsTypedDict": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailRequest": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailRequestBody": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailRequestBodyTypedDict": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailRequestTypedDict": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailResponse": ".bulkassignmemberstoguardrail",
    "BulkAssignMembersToGuardrailResponseTypedDict": ".bulkassignmemberstoguardrail",
    "BulkUnassignKeysFromGuardrailGlobals": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailGlobalsTypedDict": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailRequest": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailRequestBody": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailRequestBodyTypedDict": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailRequestTypedDict": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailResponse": ".bulkunassignkeysfromguardrail",
    "BulkUnassignKeysFromGuardrailResponseTypedDict": ".bulkunassignkeysfromguardrail",
    "BulkUnassignMembersFromGuardrailGlobals": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailGlobalsTypedDict": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailRequest": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailRequestBody": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailRequestBodyTypedDict": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailRequestTypedDict": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailResponse": ".bulkunassignmembersfromguardrail",
    "BulkUnassignMembersFromGuardrailResponseTypedDict": ".bulkunassignmembersfromguardrail",
    "CreateAuthKeysCodeCodeChallengeMethod": ".createauthkeyscode",
    "CreateAuthKeysCodeData": ".createauthkeyscode",
    "CreateAuthKeysCodeDataTypedDict": ".createauthkeyscode",
    "CreateAuthKeysCodeGlobals": ".createauthkeyscode",
    "CreateAuthKeysCodeGlobalsTypedDict": ".createauthkeyscode",
    "CreateAuthKeysCodeRequest": ".createauthkeyscode",
    "CreateAuthKeysCodeRequestBody": ".createauthkeyscode",
    "CreateAuthKeysCodeRequestBodyTypedDict": ".createauthkeyscode",
    "CreateAuthKeysCodeRequestTypedDict": ".createauthkeyscode",
    "CreateAuthKeysCodeResponse": ".createauthkeyscode",
    "CreateAuthKeysCodeResponseTypedDict": ".createauthkeyscode",
    "UsageLimitType": ".createauthkeyscode",
    "CallData": ".createcoinbasecharge",
    "CallDataTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeData": ".createcoinbasecharge",
    "CreateCoinbaseChargeDataTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeGlobals": ".createcoinbasecharge",
    "CreateCoinbaseChargeGlobalsTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeRequest": ".createcoinbasecharge",
    "CreateCoinbaseChargeRequestTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeResponse": ".createcoinbasecharge",
    "CreateCoinbaseChargeResponseTypedDict": ".createcoinbasecharge",
    "CreateCoinbaseChargeSecurity": ".createcoinbasecharge",
    "CreateCoinbaseChargeSecurityTypedDict": ".createcoinbasecharge",
    "Metadata": ".createcoinbasecharge",
    "MetadataTypedDict": ".createcoinbasecharge",
    "TransferIntent": ".createcoinbasecharge",
    "TransferIntentTypedDict": ".createcoinbasecharge",
    "Web3Data": ".createcoinbasecharge",
    "Web3DataTypedDict": ".createcoinbasecharge",
    "Content": ".createembeddings",
    "ContentImageURL": ".createembeddings",
    "ContentImageURLTypedDict": ".createembeddings",
    "ContentText": ".createembeddings",
    "ContentTextTypedDict": ".createembeddings",
    "ContentTypedDict": ".createembeddings",
    "CreateEmbeddingsData": ".createembeddings",
    "CreateEmbeddingsDataTypedDict": ".createembeddings",
    "CreateEmbeddingsGlobals": ".createembeddings",
    "CreateEmbeddingsGlobalsTypedDict": ".createembeddings",
    "CreateEmbeddingsRequest": ".createembeddings",
    "CreateEmbeddingsRequestBody": ".createembeddings",
    "CreateEmbeddingsRequestBodyTypedDict": ".createembeddings",
    "CreateEmbeddingsRequestTypedDict": ".createembeddings",
    "CreateEmbeddingsResponse": ".createembeddings",
    "CreateEmbeddingsResponseBody": ".createembeddings",
    "CreateEmbeddingsResponseBodyTypedDict": ".createembeddings",
    "CreateEmbeddingsResponseTypedDict": ".createembeddings",
    "Embedding": ".createembeddings",
    "EmbeddingTypedDict": ".createembeddings",
    "EncodingFormat": ".createembeddings",
    "ImageURL": ".createembeddings",
    "ImageURLTypedDict": ".createembeddings",
    "Input": ".createembeddings",
    "InputTypedDict": ".createembeddings",
    "InputUnion": ".createembeddings",
    "InputUnionTypedDict": ".createembeddings",
    "Object": ".createembeddings",
    "ObjectEmbedding": ".createembeddings",
    "TypeImageURL": ".createembeddings",
    "TypeText": ".createembeddings",
    "Usage": ".createembeddings",
    "UsageTypedDict": ".createembeddings",
    "CreateGuardrailData": ".createguardrail",
    "CreateGuardrailDataTypedDict": ".createguardrail",
    "CreateGuardrailGlobals": ".createguardrail",
    "CreateGuardrailGlobalsTypedDict": ".createguardrail",
    "CreateGuardrailRequest": ".createguardrail",
    "CreateGuardrailRequestBody": ".createguardrail",
    "CreateGuardrailRequestBodyTypedDict": ".createguardrail",
    "CreateGuardrailRequestTypedDict": ".createguardrail",
    "CreateGuardrailResetIntervalRequest": ".createguardrail",
    "CreateGuardrailResetIntervalResponse": ".createguardrail",
    "CreateGuardrailResponse": ".createguardrail",
    "CreateGuardrailResponseTypedDict": ".createguardrail",
    "CreateKeysData": ".createkeys",
    "CreateKeysDataTypedDict": ".createkeys",
    "CreateKeysGlobals": ".createkeys",
    "CreateKeysGlobalsTypedDict": ".createkeys",
    "CreateKeysLimitReset": ".createkeys",
    "CreateKeysRequest": ".createkeys",
    "CreateKeysRequestBody": ".createkeys",
    "CreateKeysRequestBodyTypedDict": ".createkeys",
    "CreateKeysRequestTypedDict": ".createkeys",
    "CreateKeysResponse": ".createkeys",
    "CreateKeysResponseTypedDict": ".createkeys",
    "CreateResponsesGlobals": ".createresponses",
    "CreateResponsesGlobalsTypedDict": ".createresponses",
    "CreateResponsesRequest": ".createresponses",
    "CreateResponsesRequestTypedDict": ".createresponses",
    "CreateResponsesResponse": ".createresponses",
    "CreateResponsesResponseBody": ".createresponses",
    "CreateResponsesResponseBodyTypedDict": ".createresponses",
    "CreateResponsesResponseTypedDict": ".createresponses",
    "DeleteGuardrailGlobals": ".deleteguardrail",
    "DeleteGuardrailGlobalsTypedDict": ".deleteguardrail",
    "DeleteGuardrailRequest": ".deleteguardrail",
    "DeleteGuardrailRequestTypedDict": ".deleteguardrail",
    "DeleteGuardrailResponse": ".deleteguardrail",
    "DeleteGuardrailResponseTypedDict": ".deleteguardrail",
    "DeleteKeysGlobals": ".deletekeys",
    "DeleteKeysGlobalsTypedDict": ".deletekeys",
    "DeleteKeysRequest": ".deletekeys",
    "DeleteKeysRequestTypedDict": ".deletekeys",
    "DeleteKeysResponse": ".deletekeys",
    "DeleteKeysResponseTypedDict": ".deletekeys",
    "ExchangeAuthCodeForAPIKeyCodeChallengeMethod": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyGlobals": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyGlobalsTypedDict": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyRequest": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyRequestBody": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyRequestBodyTypedDict": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyRequestTypedDict": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyResponse": ".exchangeauthcodeforapikey",
    "ExchangeAuthCodeForAPIKeyResponseTypedDict": ".exchangeauthcodeforapikey",
    "GetCreditsData": ".getcredits",
    "GetCreditsDataTypedDict": ".getcredits",
    "GetCreditsGlobals": ".getcredits",
    "GetCreditsGlobalsTypedDict": ".getcredits",
    "GetCreditsRequest": ".getcredits",
    "GetCreditsRequestTypedDict": ".getcredits",
    "GetCreditsResponse": ".getcredits",
    "GetCreditsResponseTypedDict": ".getcredits",
    "GetCurrentKeyData": ".getcurrentkey",
    "GetCurrentKeyDataTypedDict": ".getcurrentkey",
    "GetCurrentKeyGlobals": ".getcurrentkey",
    "GetCurrentKeyGlobalsTypedDict": ".getcurrentkey",
    "GetCurrentKeyRequest": ".getcurrentkey",
    "GetCurrentKeyRequestTypedDict": ".getcurrentkey",
    "GetCurrentKeyResponse": ".getcurrentkey",
    "GetCurrentKeyResponseTypedDict": ".getcurrentkey",
    "RateLimit": ".getcurrentkey",
    "RateLimitTypedDict": ".getcurrentkey",
    "APIType": ".getgeneration",
    "GetGenerationData": ".getgeneration",
    "GetGenerationDataTypedDict": ".getgeneration",
    "GetGenerationGlobals": ".getgeneration",
    "GetGenerationGlobalsTypedDict": ".getgeneration",
    "GetGenerationRequest": ".getgeneration",
    "GetGenerationRequestTypedDict": ".getgeneration",
    "GetGenerationResponse": ".getgeneration",
    "GetGenerationResponseTypedDict": ".getgeneration",
    "ProviderName": ".getgeneration",
    "ProviderResponse": ".getgeneration",
    "ProviderResponseTypedDict": ".getgeneration",
    "GetGuardrailData": ".getguardrail",
    "GetGuardrailDataTypedDict": ".getguardrail",
    "GetGuardrailGlobals": ".getguardrail",
    "GetGuardrailGlobalsTypedDict": ".getguardrail",
    "GetGuardrailRequest": ".getguardrail",
    "GetGuardrailRequestTypedDict": ".getguardrail",
    "GetGuardrailResetInterval": ".getguardrail",
    "GetGuardrailResponse": ".getguardrail",
    "GetGuardrailResponseTypedDict": ".getguardrail",
    "GetKeyData": ".getkey",
    "GetKeyDataTypedDict": ".getkey",
    "GetKeyGlobals": ".getkey",
    "GetKeyGlobalsTypedDict": ".getkey",
    "GetKeyRequest": ".getkey",
    "GetKeyRequestTypedDict": ".getkey",
    "GetKeyResponse": ".getkey",
    "GetKeyResponseTypedDict": ".getkey",
    "Category": ".getmodels",
    "GetModelsGlobals": ".getmodels",
    "GetModelsGlobalsTypedDict": ".getmodels",
    "GetModelsRequest": ".getmodels",
    "GetModelsRequestTypedDict": ".getmodels",
    "GetUserActivityGlobals": ".getuseractivity",
    "GetUserActivityGlobalsTypedDict": ".getuseractivity",
    "GetUserActivityRequest": ".getuseractivity",
    "GetUserActivityRequestTypedDict": ".getuseractivity",
    "GetUserActivityResponse": ".getuseractivity",
    "GetUserActivityResponseTypedDict": ".getuseractivity",
    "ListData": ".list",
    "ListDataTypedDict": ".list",
    "ListGlobals": ".list",
    "ListGlobalsTypedDict": ".list",
    "ListRequest": ".list",
    "ListRequestTypedDict": ".list",
    "ListResponse": ".list",
    "ListResponseTypedDict": ".list",
    "ListEmbeddingsModelsGlobals": ".listembeddingsmodels",
    "ListEmbeddingsModelsGlobalsTypedDict": ".listembeddingsmodels",
    "ListEmbeddingsModelsRequest": ".listembeddingsmodels",
    "ListEmbeddingsModelsRequestTypedDict": ".listembeddingsmodels",
    "ListEndpointsGlobals": ".listendpoints",
    "ListEndpointsGlobalsTypedDict": ".listendpoints",
    "ListEndpointsRequest": ".listendpoints",
    "ListEndpointsRequestTypedDict": ".listendpoints",
    "ListEndpointsResponse": ".listendpoints",
    "ListEndpointsResponseTypedDict": ".listendpoints",
    "ListEndpointsZdrGlobals": ".listendpointszdr",
    "ListEndpointsZdrGlobalsTypedDict": ".listendpointszdr",
    "ListEndpointsZdrRequest": ".listendpointszdr",
    "ListEndpointsZdrRequestTypedDict": ".listendpointszdr",
    "ListEndpointsZdrResponse": ".listendpointszdr",
    "ListEndpointsZdrResponseTypedDict": ".listendpointszdr",
    "ListGuardrailKeyAssignmentsData": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsDataTypedDict": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsGlobals": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsGlobalsTypedDict": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsRequest": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsRequestTypedDict": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsResponse": ".listguardrailkeyassignments",
    "ListGuardrailKeyAssignmentsResponseTypedDict": ".listguardrailkeyassignments",
    "ListGuardrailMemberAssignmentsData": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsDataTypedDict": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsGlobals": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsGlobalsTypedDict": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsRequest": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsRequestTypedDict": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsResponse": ".listguardrailmemberassignments",
    "ListGuardrailMemberAssignmentsResponseTypedDict": ".listguardrailmemberassignments",
    "ListGuardrailsData": ".listguardrails",
    "ListGuardrailsDataTypedDict": ".listguardrails",
    "ListGuardrailsGlobals": ".listguardrails",
    "ListGuardrailsGlobalsTypedDict": ".listguardrails",
    "ListGuardrailsRequest": ".listguardrails",
    "ListGuardrailsRequestTypedDict": ".listguardrails",
    "ListGuardrailsResetInterval": ".listguardrails",
    "ListGuardrailsResponse": ".listguardrails",
    "ListGuardrailsResponseTypedDict": ".listguardrails",
    "ListKeyAssignmentsData": ".listkeyassignments",
    "ListKeyAssignmentsDataTypedDict": ".listkeyassignments",
    "ListKeyAssignmentsGlobals": ".listkeyassignments",
    "ListKeyAssignmentsGlobalsTypedDict": ".listkeyassignments",
    "ListKeyAssignmentsRequest": ".listkeyassignments",
    "ListKeyAssignmentsRequestTypedDict": ".listkeyassignments",
    "ListKeyAssignmentsResponse": ".listkeyassignments",
    "ListKeyAssignmentsResponseTypedDict": ".listkeyassignments",
    "ListMemberAssignmentsData": ".listmemberassignments",
    "ListMemberAssignmentsDataTypedDict": ".listmemberassignments",
    "ListMemberAssignmentsGlobals": ".listmemberassignments",
    "ListMemberAssignmentsGlobalsTypedDict": ".listmemberassignments",
    "ListMemberAssignmentsRequest": ".listmemberassignments",
    "ListMemberAssignmentsRequestTypedDict": ".listmemberassignments",
    "ListMemberAssignmentsResponse": ".listmemberassignments",
    "ListMemberAssignmentsResponseTypedDict": ".listmemberassignments",
    "ListModelsCountGlobals": ".listmodelscount",
    "ListModelsCountGlobalsTypedDict": ".listmodelscount",
    "ListModelsCountRequest": ".listmodelscount",
    "ListModelsCountRequestTypedDict": ".listmodelscount",
    "ListModelsUserGlobals": ".listmodelsuser",
    "ListModelsUserGlobalsTypedDict": ".listmodelsuser",
    "ListModelsUserRequest": ".listmodelsuser",
    "ListModelsUserRequestTypedDict": ".listmodelsuser",
    "ListModelsUserSecurity": ".listmodelsuser",
    "ListModelsUserSecurityTypedDict": ".listmodelsuser",
    "Datacenter": ".listproviders",
    "Headquarters": ".listproviders",
    "ListProvidersData": ".listproviders",
    "ListProvidersDataTypedDict": ".listproviders",
    "ListProvidersGlobals": ".listproviders",
    "ListProvidersGlobalsTypedDict": ".listproviders",
    "ListProvidersRequest": ".listproviders",
    "ListProvidersRequestTypedDict": ".listproviders",
    "ListProvidersResponse": ".listproviders",
    "ListProvidersResponseTypedDict": ".listproviders",
    "SendChatCompletionRequestGlobals": ".sendchatcompletionrequest",
    "SendChatCompletionRequestGlobalsTypedDict": ".sendchatcompletionrequest",
    "SendChatCompletionRequestRequest": ".sendchatcompletionrequest",
    "SendChatCompletionRequestRequestTypedDict": ".sendchatcompletionrequest",
    "SendChatCompletionRequestResponse": ".sendchatcompletionrequest",
    "SendChatCompletionRequestResponseBody": ".sendchatcompletionrequest",
    "SendChatCompletionRequestResponseBodyTypedDict": ".sendchatcompletionrequest",
    "SendChatCompletionRequestResponseTypedDict": ".sendchatcompletionrequest",
    "UpdateGuardrailData": ".updateguardrail",
    "UpdateGuardrailDataTypedDict": ".updateguardrail",
    "UpdateGuardrailGlobals": ".updateguardrail",
    "UpdateGuardrailGlobalsTypedDict": ".updateguardrail",
    "UpdateGuardrailRequest": ".updateguardrail",
    "UpdateGuardrailRequestBody": ".updateguardrail",
    "UpdateGuardrailRequestBodyTypedDict": ".updateguardrail",
    "UpdateGuardrailRequestTypedDict": ".updateguardrail",
    "UpdateGuardrailResetIntervalRequest": ".updateguardrail",
    "UpdateGuardrailResetIntervalResponse": ".updateguardrail",
    "UpdateGuardrailResponse": ".updateguardrail",
    "UpdateGuardrailResponseTypedDict": ".updateguardrail",
    "UpdateKeysData": ".updatekeys",
    "UpdateKeysDataTypedDict": ".updatekeys",
    "UpdateKeysGlobals": ".updatekeys",
    "UpdateKeysGlobalsTypedDict": ".updatekeys",
    "UpdateKeysLimitReset": ".updatekeys",
    "UpdateKeysRequest": ".updatekeys",
    "UpdateKeysRequestBody": ".updatekeys",
    "UpdateKeysRequestBodyTypedDict": ".updatekeys",
    "UpdateKeysRequestTypedDict": ".updatekeys",
    "UpdateKeysResponse": ".updatekeys",
    "UpdateKeysResponseTypedDict": ".updatekeys",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import (
    FieldMetadata,
    HeaderMetadata,
    QueryParamMetadata,
    validate_open_enum,
)
import pydantic
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class GetModelsGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class GetModelsGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


Category = Union[
    Literal[
        "programming",
        "roleplay",
        "marketing",
        "marketing/seo",
        "technology",
        "science",
        "translation",
        "legal",
        "finance",
        "health",
        "trivia",
        "academia",
    ],
    UnrecognizedStr,
]
r"""Filter models by use case category"""


class GetModelsRequestTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """
    category: NotRequired[Category]
    r"""Filter models by use case category"""
    supported_parameters: NotRequired[str]
    output_modalities: NotRequired[str]
    r"""Filter models by output modality. Accepts a comma-separated list of modalities (text, image, audio, embeddings) or \"all\" to include all models. Defaults to \"text\"."""


class GetModelsRequest(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """

    category: Annotated[
        Annotated[Optional[Category], PlainValidator(validate_open_enum(False))],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter models by use case category"""

    supported_parameters: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    output_modalities: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter models by output modality. Accepts a comma-separated list of modalities (text, image, audio, embeddings) or \"all\" to include all models. Defaults to \"text\"."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .videoinput import VideoInput, VideoInputTypedDict
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatMessageContentItemVideoType = Literal["video_url",]


class ChatMessageContentItemVideoTypedDict(TypedDict):
    r"""Video input content part"""

    type: ChatMessageContentItemVideoType
    video_url: VideoInputTypedDict
    r"""Video input object"""


class ChatMessageContentItemVideo(BaseModel):
    r"""Video input content part"""

    type: ChatMessageContentItemVideoType

    video_url: VideoInput
    r"""Video input object"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatassistantmessage import ChatAssistantMessage, ChatAssistantMessageTypedDict
from .chatdevelopermessage import ChatDeveloperMessage, ChatDeveloperMessageTypedDict
from .chatsystemmessage import ChatSystemMessage, ChatSystemMessageTypedDict
from .chattoolmessage import ChatToolMessage, ChatToolMessageTypedDict
from .chatusermessage import ChatUserMessage, ChatUserMessageTypedDict
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ChatMessagesTypedDict = TypeAliasType(
    "ChatMessagesTypedDict",
    Union[
        ChatSystemMessageTypedDict,
        ChatUserMessageTypedDict,
        ChatDeveloperMessageTypedDict,
        ChatToolMessageTypedDict,
        ChatAssistantMessageTypedDict,
    ],
)
r"""Chat completion message with role-based discrimination"""


ChatMessages = Annotated[
    Union[
        Annotated[ChatSystemMessage, Tag("system")],
        Annotated[ChatUserMessage, Tag("user")],
        Annotated[ChatDeveloperMessage, Tag("developer")],
        Annotated[ChatAssistantMessage, Tag("assistant")],
        Annotated[ChatToolMessage, Tag("tool")],
    ],
    Discriminator(lambda m: get_discriminator(m, "role", "role")),
]
r"""Chat completion message with role-based discrimination"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatdebugoptions import ChatDebugOptions, ChatDebugOptionsTypedDict
from .chatformatgrammarconfig import (
    ChatFormatGrammarConfig,
    ChatFormatGrammarConfigTypedDict,
)
from .chatformatjsonschemaconfig import (
    ChatFormatJSONSchemaConfig,
    ChatFormatJSONSchemaConfigTypedDict,
)
from .chatformatpythonconfig import (
    ChatFormatPythonConfig,
    ChatFormatPythonConfigTypedDict,
)
from .chatformattextconfig import ChatFormatTextConfig, ChatFormatTextConfigTypedDict
from .chatfunctiontool import ChatFunctionTool, ChatFunctionToolTypedDict
from .chatmessages import ChatMessages, ChatMessagesTypedDict
from .chatstreamoptions import ChatStreamOptions, ChatStreamOptionsTypedDict
from .chattoolchoice import ChatToolChoice, ChatToolChoiceTypedDict
from .contextcompressionengine import ContextCompressionEngine
from .datacollection import DataCollection
from .formatjsonobjectconfig import (
    FormatJSONObjectConfig,
    FormatJSONObjectConfigTypedDict,
)
from .pdfparseroptions import PDFParserOptions, PDFParserOptionsTypedDict
from .preferredmaxlatency import PreferredMaxLatency, PreferredMaxLatencyTypedDict
from .preferredminthroughput import (
    PreferredMinThroughput,
    PreferredMinThroughputTypedDict,
)
from .providername import ProviderName
from .quantization import Quantization
from .websearchengine import WebSearchEngine
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_open_enum
import pydantic
from pydantic import ConfigDict, Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ChatRequestOrderTypedDict = TypeAliasType(
    "ChatRequestOrderTypedDict", Union[ProviderName, str]
)


ChatRequestOrder = TypeAliasType(
    "ChatRequestOrder",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ChatRequestOnlyTypedDict = TypeAliasType(
    "ChatRequestOnlyTypedDict", Union[ProviderName, str]
)


ChatRequestOnly = TypeAliasType(
    "ChatRequestOnly",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ChatRequestIgnoreTypedDict = TypeAliasType(
    "ChatRequestIgnoreTypedDict", Union[ProviderName, str]
)


ChatRequestIgnore = TypeAliasType(
    "ChatRequestIgnore",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


ChatRequestSortEnum = Union[
    Literal[
        "price",
        "throughput",
        "latency",
        "exacto",
    ],
    UnrecognizedStr,
]


ChatRequestProviderSortConfigEnum = Literal[
    "price",
    "throughput",
    "latency",
    "exacto",
]


ChatRequestBy = Union[
    Literal[
        "price",
        "throughput",
        "latency",
        "exacto",
    ],
    UnrecognizedStr,
]
r"""The provider sorting strategy (price, throughput, latency)"""


ChatRequestPartition = Union[
    Literal[
        "model",
        "none",
    ],
    UnrecognizedStr,
]
r"""Partitioning strategy for sorting: \"model\" (default) groups endpoints by model before sorting (fallback models remain fallbacks), \"none\" sorts all endpoints together regardless of model."""


class ChatRequestProviderSortConfigTypedDict(TypedDict):
    by: NotRequired[Nullable[ChatRequestBy]]
    r"""The provider sorting strategy (price, throughput, latency)"""
    partition: NotRequired[Nullable[ChatRequestPartition]]
    r"""Partitioning strategy for sorting: \"model\" (default) groups endpoints by model before sorting (fallback models remain fallbacks), \"none\" sorts all endpoints together regardless of model."""


class ChatRequestProviderSortConfig(BaseModel):
    by: Annotated[
        OptionalNullable[ChatRequestBy], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The provider sorting strategy (price, throughput, latency)"""

    partition: Annotated[
        OptionalNullable[ChatRequestPartition],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET
    r"""Partitioning strategy for sorting: \"model\" (default) groups endpoints by model before sorting (fallback models remain fallbacks), \"none\" sorts all endpoints together regardless of model."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["by", "partition"]
        nullable_fields = ["by", "partition"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ChatRequestProviderSortConfigUnionTypedDict = TypeAliasType(
    "ChatRequestProviderSortConfigUnionTypedDict",
    Union[ChatRequestProviderSortConfigTypedDict, ChatRequestProviderSortConfigEnum],
)


ChatRequestProviderSortConfigUnion = TypeAliasType(
    "ChatRequestProviderSortConfigUnion",
    Union[ChatRequestProviderSortConfig, ChatRequestProviderSortConfigEnum],
)


ChatRequestProviderSort = Union[
    Literal[
        "price",
        "throughput",
        "latency",
        "exacto",
    ],
    UnrecognizedStr,
]
r"""The provider sorting strategy (price, throughput, latency)"""


ChatRequestSortUnionTypedDict = TypeAliasType(
    "ChatRequestSortUnionTypedDict",
    Union[
        ChatRequestProviderSort,
        ChatRequestProviderSortConfigUnionTypedDict,
        ChatRequestSortEnum,
    ],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


ChatRequestSortUnion = TypeAliasType(
    "ChatRequestSortUnion",
    Union[
        Annotated[ChatRequestProviderSort, PlainValidator(validate_open_enum(False))],
        ChatRequestProviderSortConfigUnion,
        Annotated[ChatRequestSortEnum, PlainValidator(validate_open_enum(False))],
    ],
)
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


class ChatRequestMaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[str]
    r"""Price per million prompt tokens"""
    completion: NotRequired[str]
    image: NotRequired[str]
    audio: NotRequired[str]
    request: NotRequired[str]


class ChatRequestMaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[str] = None
    r"""Price per million prompt tokens"""

    completion: Optional[str] = None

    image: Optional[str] = None

    audio: Optional[str] = None

    request: Optional[str] = None


class ChatRequestProviderTypedDict(TypedDict):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[DataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """
    zdr: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""
    enforce_distillable_text: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""
    order: NotRequired[Nullable[List[ChatRequestOrderTypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[ChatRequestOnlyTypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[ChatRequestIgnoreTypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[Quantization]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[ChatRequestSortUnionTypedDict]]
    max_price: NotRequired[ChatRequestMaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""
    preferred_min_throughput: NotRequired[Nullable[PreferredMinThroughputTypedDict]]
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""
    preferred_max_latency: NotRequired[Nullable[PreferredMaxLatencyTypedDict]]
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""


class ChatRequestProvider(BaseModel):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[DataCollection], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it

    - deny: use only providers which do not collect user data.
    """

    zdr: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""

    enforce_distillable_text: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""

    order: OptionalNullable[List[ChatRequestOrder]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[ChatRequestOnly]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[ChatRequestIgnore]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[Annotated[Quantization, PlainValidator(validate_open_enum(False))]]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: OptionalNullable[ChatRequestSortUnion] = UNSET

    max_price: Optional[ChatRequestMaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    preferred_min_throughput: OptionalNullable[PreferredMinThroughput] = UNSET
    r"""Preferred minimum throughput (in tokens per second). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints below the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    preferred_max_latency: OptionalNullable[PreferredMaxLatency] = UNSET
    r"""Preferred maximum latency (in seconds). Can be a number (applies to p50) or an object with percentile-specific cutoffs. Endpoints above the threshold(s) may still be used, but are deprioritized in routing. When using fallback models, this may cause a fallback model to be used instead of the primary model if it meets the threshold."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "preferred_min_throughput",
            "preferred_max_latency",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ChatRequestIDContextCompression = Literal["context-compression",]


class ChatRequestPluginContextCompressionTypedDict(TypedDict):
    id: ChatRequestIDContextCompression
    enabled: NotRequired[bool]
    r"""Set to false to disable the context-compression plugin for this request. Defaults to true."""
    engine: NotRequired[ContextCompressionEngine]
    r"""The compression engine to use. Defaults to \"middle-out\"."""


class ChatRequestPluginContextCompression(BaseModel):
    id: ChatRequestIDContextCompression

    enabled: Optional[bool] = None
    r"""Set to false to disable the context-compression plugin for this request. Defaults to true."""

    engine: Optional[ContextCompressionEngine] = None
    r"""The compression engine to use. Defaults to \"middle-out\"."""


ChatRequestIDResponseHealing = Literal["response-healing",]


class ChatRequestPluginResponseHealingTypedDict(TypedDict):
    id: ChatRequestIDResponseHealing
    enabled: NotRequired[bool]
    r"""Set to false to disable the response-healing plugin for this request. Defaults to true."""


class ChatRequestPluginResponseHealing(BaseModel):
    id: ChatRequestIDResponseHealing

    enabled: Optional[bool] = None
    r"""Set to false to disable the response-healing plugin for this request. Defaults to true."""


ChatRequestIDFileParser = Literal["file-parser",]


class ChatRequestPluginFileParserTypedDict(TypedDict):
    id: ChatRequestIDFileParser
    enabled: NotRequired[bool]
    r"""Set to false to disable the file-parser plugin for this request. Defaults to true."""
    pdf: NotRequired[PDFParserOptionsTypedDict]
    r"""Options for PDF parsing."""


class ChatRequestPluginFileParser(BaseModel):
    id: ChatRequestIDFileParser

    enabled: Optional[bool] = None
    r"""Set to false to disable the file-parser plugin for this request. Defaults to true."""

    pdf: Optional[PDFParserOptions] = None
    r"""Options for PDF parsing."""


ChatRequestIDWeb = Literal["web",]


class ChatRequestPluginWebTypedDict(TypedDict):
    id: ChatRequestIDWeb
    enabled: NotRequired[bool]
    r"""Set to false to disable the web-search plugin for this request. Defaults to true."""
    max_results: NotRequired[float]
    search_prompt: NotRequired[str]
    engine: NotRequired[WebSearchEngine]
    r"""The search engine to use for web search."""
    include_domains: NotRequired[List[str]]
    r"""A list of domains to restrict web search results to. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""
    exclude_domains: NotRequired[List[str]]
    r"""A list of domains to exclude from web search results. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""


class ChatRequestPluginWeb(BaseModel):
    id: ChatRequestIDWeb

    enabled: Optional[bool] = None
    r"""Set to false to disable the web-search plugin for this request. Defaults to true."""

    max_results: Optional[float] = None

    search_prompt: Optional[str] = None

    engine: Annotated[
        Optional[WebSearchEngine], PlainValidator(validate_open_enum(False))
    ] = None
    r"""The search engine to use for web search."""

    include_domains: Optional[List[str]] = None
    r"""A list of domains to restrict web search results to. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""

    exclude_domains: Optional[List[str]] = None
    r"""A list of domains to exclude from web search results. Supports wildcards (e.g. \"*.substack.com\") and path filtering (e.g. \"openai.com/blog\")."""


ChatRequestIDModeration = Literal["moderation",]


class ChatRequestPluginModerationTypedDict(TypedDict):
    id: ChatRequestIDModeration


class ChatRequestPluginModeration(BaseModel):
    id: ChatRequestIDModeration


ChatRequestIDAutoRouter = Literal["auto-router",]


class ChatRequestPluginAutoRouterTypedDict(TypedDict):
    id: ChatRequestIDAutoRouter
    enabled: NotRequired[bool]
    r"""Set to false to disable the auto-router plugin for this request. Defaults to true."""
    allowed_models: NotRequired[List[str]]
    r"""List of model patterns to filter which models the auto-router can route between. Supports wildcards (e.g., \"anthropic/*\" matches all Anthropic models). When not specified, uses the default supported models list."""


class ChatRequestPluginAutoRouter(BaseModel):
    id: ChatRequestIDAutoRouter

    enabled: Optional[bool] = None
    r"""Set to false to disable the auto-router plugin for this request. Defaults to true."""

    allowed_models: Optional[List[str]] = None
    r"""List of model patterns to filter which models the auto-router can route between. Supports wildcards (e.g., \"anthropic/*\" matches all Anthropic models). When not specified, uses the default supported models list."""


ChatRequestPluginUnionTypedDict = TypeAliasType(
    "ChatRequestPluginUnionTypedDict",
    Union[
        ChatRequestPluginModerationTypedDict,
        ChatRequestPluginResponseHealingTypedDict,
        ChatRequestPluginAutoRouterTypedDict,
        ChatRequestPluginFileParserTypedDict,
        ChatRequestPluginContextCompressionTypedDict,
        ChatRequestPluginWebTypedDict,
    ],
)


ChatRequestPluginUnion = Annotated[
    Union[
        Annotated[ChatRequestPluginAutoRouter, Tag("auto-router")],
        Annotated[ChatRequestPluginModeration, Tag("moderation")],
        Annotated[ChatRequestPluginWeb, Tag("web")],
        Annotated[ChatRequestPluginFileParser, Tag("file-parser")],
        Annotated[ChatRequestPluginResponseHealing, Tag("response-healing")],
        Annotated[ChatRequestPluginContextCompression, Tag("context-compression")],
    ],
    Discriminator(lambda m: get_discriminator(m, "id", "id")),
]


class ChatRequestTraceTypedDict(TypedDict):
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""

    trace_id: NotRequired[str]
    trace_name: NotRequired[str]
    span_name: NotRequired[str]
    generation_name: NotRequired[str]
    parent_span_id: NotRequired[str]


class ChatRequestTrace(BaseModel):
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""

    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Nullable[Any]] = pydantic.Field(init=False)

    trace_id: Optional[str] = None

    trace_name: Optional[str] = None

    span_name: Optional[str] = None

    generation_name: Optional[str] = None

    parent_span_id: Optional[str] = None

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]


Effort = Union[
    Literal[
        "xhigh",
        "high",
        "medium",
        "low",
        "minimal",
        "none",
    ],
    UnrecognizedStr,
]
r"""Constrains effort on reasoning for reasoning models"""


class ReasoningTypedDict(TypedDict):
    r"""Configuration options for reasoning models"""

    effort: NotRequired[Nullable[Effort]]
    r"""Constrains effort on reasoning for reasoning models"""
    summary: NotRequired[Nullable[Any]]


class Reasoning(BaseModel):
    r"""Configuration options for reasoning models"""

    effort: Annotated[
        OptionalNullable[Effort], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Constrains effort on reasoning for reasoning models"""

    summary: OptionalNullable[Any] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["effort", "summary"]
        nullable_fields = ["effort", "summary"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


ResponseFormatTypedDict = TypeAliasType(
    "ResponseFormatTypedDict",
    Union[
        ChatFormatTextConfigTypedDict,
        FormatJSONObjectConfigTypedDict,
        ChatFormatPythonConfigTypedDict,
        ChatFormatJSONSchemaConfigTypedDict,
        ChatFormatGrammarConfigTypedDict,
    ],
)
r"""Response format configuration"""


ResponseFormat = Annotated[
    Union[
        Annotated[ChatFormatTextConfig, Tag("text")],
        Annotated[FormatJSONObjectConfig, Tag("json_object")],
        Annotated[ChatFormatJSONSchemaConfig, Tag("json_schema")],
        Annotated[ChatFormatGrammarConfig, Tag("grammar")],
        Annotated[ChatFormatPythonConfig, Tag("python")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""Response format configuration"""


StopTypedDict = TypeAliasType("StopTypedDict", Union[str, List[str], Any])
r"""Stop sequences (up to 4)"""


Stop = TypeAliasType("Stop", Union[str, List[str], Any])
r"""Stop sequences (up to 4)"""


ChatRequestImageConfigTypedDict = TypeAliasType(
    "ChatRequestImageConfigTypedDict", Union[str, float, List[Nullable[Any]]]
)


ChatRequestImageConfig = TypeAliasType(
    "ChatRequestImageConfig", Union[str, float, List[Nullable[Any]]]
)


Modality = Union[
    Literal[
        "text",
        "image",
        "audio",
    ],
    UnrecognizedStr,
]


ChatRequestType = Literal["ephemeral",]


ChatRequestTTL = Union[
    Literal[
        "5m",
        "1h",
    ],
    UnrecognizedStr,
]


class CacheControlTypedDict(TypedDict):
    r"""Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models."""

    type: ChatRequestType
    ttl: NotRequired[ChatRequestTTL]


class CacheControl(BaseModel):
    r"""Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models."""

    type: ChatRequestType

    ttl: Annotated[
        Optional[ChatRequestTTL], PlainValidator(validate_open_enum(False))
    ] = None


ChatRequestServiceTier = Union[
    Literal[
        "auto",
        "default",
        "flex",
        "priority",
        "scale",
    ],
    UnrecognizedStr,
]
r"""The service tier to use for processing this request."""


class ChatRequestTypedDict(TypedDict):
    r"""Chat completion request parameters"""

    messages: List[ChatMessagesTypedDict]
    r"""List of messages for the conversation"""
    provider: NotRequired[Nullable[ChatRequestProviderTypedDict]]
    r"""When multiple model providers are available, optionally indicate your routing preference."""
    plugins: NotRequired[List[ChatRequestPluginUnionTypedDict]]
    r"""Plugins you want to enable for this request, including their settings."""
    user: NotRequired[str]
    r"""Unique user identifier"""
    session_id: NotRequired[str]
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters."""
    trace: NotRequired[ChatRequestTraceTypedDict]
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""
    model: NotRequired[str]
    r"""Model to use for completion"""
    models: NotRequired[List[str]]
    r"""Models to use for completion"""
    frequency_penalty: NotRequired[Nullable[float]]
    r"""Frequency penalty (-2.0 to 2.0)"""
    logit_bias: NotRequired[Nullable[Dict[str, float]]]
    r"""Token logit bias adjustments"""
    logprobs: NotRequired[Nullable[bool]]
    r"""Return log probabilities"""
    top_logprobs: NotRequired[Nullable[float]]
    r"""Number of top log probabilities to return (0-20)"""
    max_completion_tokens: NotRequired[Nullable[float]]
    r"""Maximum tokens in completion"""
    max_tokens: NotRequired[Nullable[float]]
    r"""Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16."""
    metadata: NotRequired[Dict[str, str]]
    r"""Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)"""
    presence_penalty: NotRequired[Nullable[float]]
    r"""Presence penalty (-2.0 to 2.0)"""
    reasoning: NotRequired[ReasoningTypedDict]
    r"""Configuration options for reasoning models"""
    response_format: NotRequired[ResponseFormatTypedDict]
    r"""Response format configuration"""
    seed: NotRequired[Nullable[int]]
    r"""Random seed for deterministic outputs"""
    stop: NotRequired[Nullable[StopTypedDict]]
    r"""Stop sequences (up to 4)"""
    stream: NotRequired[bool]
    r"""Enable streaming response"""
    stream_options: NotRequired[Nullable[ChatStreamOptionsTypedDict]]
    r"""Streaming configuration options"""
    temperature: NotRequired[Nullable[float]]
    r"""Sampling temperature (0-2)"""
    parallel_tool_calls: NotRequired[Nullable[bool]]
    tool_choice: NotRequired[ChatToolChoiceTypedDict]
    r"""Tool choice configuration"""
    tools: NotRequired[List[ChatFunctionToolTypedDict]]
    r"""Available tools for function calling"""
    top_p: NotRequired[Nullable[float]]
    r"""Nucleus sampling parameter (0-1)"""
    debug: NotRequired[ChatDebugOptionsTypedDict]
    r"""Debug options for inspecting request transformations (streaming only)"""
    image_config: NotRequired[Dict[str, ChatRequestImageConfigTypedDict]]
    r"""Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details."""
    modalities: NotRequired[List[Modality]]
    r"""Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\"."""
    cache_control: NotRequired[CacheControlTypedDict]
    r"""Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models."""
    service_tier: NotRequired[Nullable[ChatRequestServiceTier]]
    r"""The service tier to use for processing this request."""


class ChatRequest(BaseModel):
    r"""Chat completion request parameters"""

    messages: List[ChatMessages]
    r"""List of messages for the conversation"""

    provider: OptionalNullable[ChatRequestProvider] = UNSET
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    plugins: Optional[List[ChatRequestPluginUnion]] = None
    r"""Plugins you want to enable for this request, including their settings."""

    user: Optional[str] = None
    r"""Unique user identifier"""

    session_id: Optional[str] = None
    r"""A unique identifier for grouping related requests (e.g., a conversation or agent workflow) for observability. If provided in both the request body and the x-session-id header, the body value takes precedence. Maximum of 256 characters."""

    trace: Optional[ChatRequestTrace] = None
    r"""Metadata for observability and tracing. Known keys (trace_id, trace_name, span_name, generation_name, parent_span_id) have special handling. Additional keys are passed through as custom metadata to configured broadcast destinations."""

    model: Optional[str] = None
    r"""Model to use for completion"""

    models: Optional[List[str]] = None
    r"""Models to use for completion"""

    frequency_penalty: OptionalNullable[float] = UNSET
    r"""Frequency penalty (-2.0 to 2.0)"""

    logit_bias: OptionalNullable[Dict[str, float]] = UNSET
    r"""Token logit bias adjustments"""

    logprobs: OptionalNullable[bool] = UNSET
    r"""Return log probabilities"""

    top_logprobs: OptionalNullable[float] = UNSET
    r"""Number of top log probabilities to return (0-20)"""

    max_completion_tokens: OptionalNullable[float] = UNSET
    r"""Maximum tokens in completion"""

    max_tokens: OptionalNullable[float] = UNSET
    r"""Maximum tokens (deprecated, use max_completion_tokens). Note: some providers enforce a minimum of 16."""

    metadata: Optional[Dict[str, str]] = None
    r"""Key-value pairs for additional object information (max 16 pairs, 64 char keys, 512 char values)"""

    presence_penalty: OptionalNullable[float] = UNSET
    r"""Presence penalty (-2.0 to 2.0)"""

    reasoning: Optional[Reasoning] = None
    r"""Configuration options for reasoning models"""

    response_format: Optional[ResponseFormat] = None
    r"""Response format configuration"""

    seed: OptionalNullable[int] = UNSET
    r"""Random seed for deterministic outputs"""

    stop: OptionalNullable[Stop] = UNSET
    r"""Stop sequences (up to 4)"""

    stream: Optional[bool] = False
    r"""Enable streaming response"""

    stream_options: OptionalNullable[ChatStreamOptions] = UNSET
    r"""Streaming configuration options"""

    temperature: OptionalNullable[float] = 1
    r"""Sampling temperature (0-2)"""

    parallel_tool_calls: OptionalNullable[bool] = UNSET

    tool_choice: Optional[ChatToolChoice] = None
    r"""Tool choice configuration"""

    tools: Optional[List[ChatFunctionTool]] = None
    r"""Available tools for function calling"""

    top_p: OptionalNullable[float] = 1
    r"""Nucleus sampling parameter (0-1)"""

    debug: Optional[ChatDebugOptions] = None
    r"""Debug options for inspecting request transformations (streaming only)"""

    image_config: Optional[Dict[str, ChatRequestImageConfig]] = None
    r"""Provider-specific image configuration options. Keys and values vary by model/provider. See https://openrouter.ai/docs/guides/overview/multimodal/image-generation for more details."""

    modalities: Optional[
        List[Annotated[Modality, PlainValidator(validate_open_enum(False))]]
    ] = None
    r"""Output modalities for the response. Supported values are \"text\", \"image\", and \"audio\"."""

    cache_control: Optional[CacheControl] = None
    r"""Enable automatic prompt caching. When set, the system automatically applies cache breakpoints to the last cacheable block in the request. Currently supported for Anthropic Claude models."""

    service_tier: Annotated[
        OptionalNullable[ChatRequestServiceTier],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET
    r"""The service tier to use for processing this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "provider",
            "plugins",
            "user",
            "session_id",
            "trace",
            "model",
            "models",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "top_logprobs",
            "max_completion_tokens",
            "max_tokens",
            "metadata",
            "presence_penalty",
            "reasoning",
            "response_format",
            "seed",
            "stop",
            "stream",
            "stream_options",
            "temperature",
            "parallel_tool_calls",
            "tool_choice",
            "tools",
            "top_p",
            "debug",
            "image_config",
            "modalities",
            "cache_control",
            "service_tier",
        ]
        nullable_fields = [
            "provider",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "top_logprobs",
            "max_completion_tokens",
            "max_tokens",
            "presence_penalty",
            "seed",
            "stop",
            "stream_options",
            "temperature",
            "parallel_tool_calls",
            "top_p",
            "service_tier",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ChatMessageContentItemImageType = Literal["image_url",]


ChatMessageContentItemImageDetail = Union[
    Literal[
        "auto",
        "low",
        "high",
    ],
    UnrecognizedStr,
]
r"""Image detail level for vision models"""


class ChatMessageContentItemImageImageURLTypedDict(TypedDict):
    url: str
    r"""URL of the image (data: URLs supported)"""
    detail: NotRequired[ChatMessageContentItemImageDetail]
    r"""Image detail level for vision models"""


class ChatMessageContentItemImageImageURL(BaseModel):
    url: str
    r"""URL of the image (data: URLs supported)"""

    detail: Annotated[
        Optional[ChatMessageContentItemImageDetail],
        PlainValidator(validate_open_enum(False)),
    ] = None
    r"""Image detail level for vision models"""


class ChatMessageContentItemImageTypedDict(TypedDict):
    r"""Image content part for vision models"""

    type: ChatMessageContentItemImageType
    image_url: ChatMessageContentItemImageImageURLTypedDict


class ChatMessageContentItemImage(BaseModel):
    r"""Image content part for vision models"""

    type: ChatMessageContentItemImageType

    image_url: ChatMessageContentItemImageImageURL



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class ListGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class ListRequestTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """
    include_disabled: NotRequired[str]
    r"""Whether to include disabled API keys in the response"""
    offset: NotRequired[str]
    r"""Number of API keys to skip for pagination"""


class ListRequest(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """

    include_disabled: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Whether to include disabled API keys in the response"""

    offset: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of API keys to skip for pagination"""


class ListDataTypedDict(TypedDict):
    hash: str
    r"""Unique hash identifier for the API key"""
    name: str
    r"""Name of the API key"""
    label: str
    r"""Human-readable label for the API key"""
    disabled: bool
    r"""Whether the API key is disabled"""
    limit: Nullable[float]
    r"""Spending limit for the API key in USD"""
    limit_remaining: Nullable[float]
    r"""Remaining spending limit in USD"""
    limit_reset: Nullable[str]
    r"""Type of limit reset for the API key"""
    include_byok_in_limit: bool
    r"""Whether to include external BYOK usage in the credit limit"""
    usage: float
    r"""Total OpenRouter credit usage (in USD) for the API key"""
    usage_daily: float
    r"""OpenRouter credit usage (in USD) for the current UTC day"""
    usage_weekly: float
    r"""OpenRouter credit usage (in USD) for the current UTC week (Monday-Sunday)"""
    usage_monthly: float
    r"""OpenRouter credit usage (in USD) for the current UTC month"""
    byok_usage: float
    r"""Total external BYOK usage (in USD) for the API key"""
    byok_usage_daily: float
    r"""External BYOK usage (in USD) for the current UTC day"""
    byok_usage_weekly: float
    r"""External BYOK usage (in USD) for the current UTC week (Monday-Sunday)"""
    byok_usage_monthly: float
    r"""External BYOK usage (in USD) for current UTC month"""
    created_at: str
    r"""ISO 8601 timestamp of when the API key was created"""
    updated_at: Nullable[str]
    r"""ISO 8601 timestamp of when the API key was last updated"""
    creator_user_id: Nullable[str]
    r"""The user ID of the key creator. For organization-owned keys, this is the member who created the key. For individual users, this is the user's own ID."""
    expires_at: NotRequired[Nullable[datetime]]
    r"""ISO 8601 UTC timestamp when the API key expires, or null if no expiration"""


class ListData(BaseModel):
    hash: str
    r"""Unique hash identifier for the API key"""

    name: str
    r"""Name of the API key"""

    label: str
    r"""Human-readable label for the API key"""

    disabled: bool
    r"""Whether the API key is disabled"""

    limit: Nullable[float]
    r"""Spending limit for the API key in USD"""

    limit_remaining: Nullable[float]
    r"""Remaining spending limit in USD"""

    limit_reset: Nullable[str]
    r"""Type of limit reset for the API key"""

    include_byok_in_limit: bool
    r"""Whether to include external BYOK usage in the credit limit"""

    usage: float
    r"""Total OpenRouter credit usage (in USD) for the API key"""

    usage_daily: float
    r"""OpenRouter credit usage (in USD) for the current UTC day"""

    usage_weekly: float
    r"""OpenRouter credit usage (in USD) for the current UTC week (Monday-Sunday)"""

    usage_monthly: float
    r"""OpenRouter credit usage (in USD) for the current UTC month"""

    byok_usage: float
    r"""Total external BYOK usage (in USD) for the API key"""

    byok_usage_daily: float
    r"""External BYOK usage (in USD) for the current UTC day"""

    byok_usage_weekly: float
    r"""External BYOK usage (in USD) for the current UTC week (Monday-Sunday)"""

    byok_usage_monthly: float
    r"""External BYOK usage (in USD) for current UTC month"""

    created_at: str
    r"""ISO 8601 timestamp of when the API key was created"""

    updated_at: Nullable[str]
    r"""ISO 8601 timestamp of when the API key was last updated"""

    creator_user_id: Nullable[str]
    r"""The user ID of the key creator. For organization-owned keys, this is the member who created the key. For individual users, this is the user's own ID."""

    expires_at: OptionalNullable[datetime] = UNSET
    r"""ISO 8601 UTC timestamp when the API key expires, or null if no expiration"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["expires_at"]
        nullable_fields = [
            "limit",
            "limit_remaining",
            "limit_reset",
            "updated_at",
            "expires_at",
            "creator_user_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListResponseTypedDict(TypedDict):
    r"""List of API keys"""

    data: List[ListDataTypedDict]
    r"""List of API keys"""


class ListResponse(BaseModel):
    r"""List of API keys"""

    data: List[ListData]
    r"""List of API keys"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class ChatAssistantImagesImageURLTypedDict(TypedDict):
    url: str
    r"""URL or base64-encoded data of the generated image"""


class ChatAssistantImagesImageURL(BaseModel):
    url: str
    r"""URL or base64-encoded data of the generated image"""


class ChatAssistantImagesTypedDict(TypedDict):
    image_url: ChatAssistantImagesImageURLTypedDict


class ChatAssistantImages(BaseModel):
    image_url: ChatAssistantImagesImageURL



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
import pydantic
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


TextDoneEventType = Literal["response.output_text.done",]


class TextDoneEventTopLogprobTypedDict(TypedDict):
    r"""Alternative token with its log probability"""

    token: NotRequired[str]
    logprob: NotRequired[float]
    bytes_: NotRequired[List[float]]


class TextDoneEventTopLogprob(BaseModel):
    r"""Alternative token with its log probability"""

    token: Optional[str] = None

    logprob: Optional[float] = None

    bytes_: Annotated[Optional[List[float]], pydantic.Field(alias="bytes")] = None


class TextDoneEventLogprobTypedDict(TypedDict):
    r"""Log probability information for a token"""

    logprob: float
    token: str
    top_logprobs: NotRequired[List[TextDoneEventTopLogprobTypedDict]]
    bytes_: NotRequired[List[float]]


class TextDoneEventLogprob(BaseModel):
    r"""Log probability information for a token"""

    logprob: float

    token: str

    top_logprobs: Optional[List[TextDoneEventTopLogprob]] = None

    bytes_: Annotated[Optional[List[float]], pydantic.Field(alias="bytes")] = None


class TextDoneEventTypedDict(TypedDict):
    r"""Event emitted when text streaming is complete"""

    type: TextDoneEventType
    output_index: float
    item_id: str
    content_index: float
    text: str
    sequence_number: float
    logprobs: List[TextDoneEventLogprobTypedDict]


class TextDoneEvent(BaseModel):
    r"""Event emitted when text streaming is complete"""

    type: TextDoneEventType

    output_index: float

    item_id: str

    content_index: float

    text: str

    sequence_number: float

    logprobs: List[TextDoneEventLogprob]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesLocalShellToolType = Literal["local_shell",]


class OpenResponsesLocalShellToolTypedDict(TypedDict):
    r"""Local shell tool configuration"""

    type: OpenResponsesLocalShellToolType


class OpenResponsesLocalShellTool(BaseModel):
    r"""Local shell tool configuration"""

    type: OpenResponsesLocalShellToolType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .applypatchservertool import ApplyPatchServerTool, ApplyPatchServerToolTypedDict
from .baseinputs_union import BaseInputsUnion, BaseInputsUnionTypedDict
from .basereasoningconfig import BaseReasoningConfig, BaseReasoningConfigTypedDict
from .codeinterpreterservertool import (
    CodeInterpreterServerTool,
    CodeInterpreterServerToolTypedDict,
)
from .codexlocalshelltool import CodexLocalShellTool, CodexLocalShellToolTypedDict
from .computeruseservertool import ComputerUseServerTool, ComputerUseServerToolTypedDict
from .customtool import CustomTool, CustomToolTypedDict
from .filesearchservertool import FileSearchServerTool, FileSearchServerToolTypedDict
from .imagegenerationservertool import (
    ImageGenerationServerTool,
    ImageGenerationServerToolTypedDict,
)
from .incompletedetails import IncompleteDetails, IncompleteDetailsTypedDict
from .legacy_websearchservertool import (
    LegacyWebSearchServerTool,
    LegacyWebSearchServerToolTypedDict,
)
from .mcpservertool import McpServerTool, McpServerToolTypedDict
from .openairesponsesresponsestatus import OpenAIResponsesResponseStatus
from .openairesponsestoolchoice_union import (
    OpenAIResponsesToolChoiceUnion,
    OpenAIResponsesToolChoiceUnionTypedDict,
)
from .outputitems import OutputItems, OutputItemsTypedDict
from .preview_20250311_websearchservertool import (
    Preview20250311WebSearchServerTool,
    Preview20250311WebSearchServerToolTypedDict,
)
from .preview_websearchservertool import (
    PreviewWebSearchServerTool,
    PreviewWebSearchServerToolTypedDict,
)
from .responseserrorfield import ResponsesErrorField, ResponsesErrorFieldTypedDict
from .shellservertool import ShellServerTool, ShellServerToolTypedDict
from .storedprompttemplate import StoredPromptTemplate, StoredPromptTemplateTypedDict
from .textconfig import TextConfig, TextConfigTypedDict
from .truncationenum import TruncationEnum
from .usage import Usage, UsageTypedDict
from .websearchservertool import WebSearchServerTool, WebSearchServerToolTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenResponsesResultObject = Literal["response",]


OpenResponsesResultType = Literal["function",]


class OpenResponsesResultToolFunctionTypedDict(TypedDict):
    r"""Function tool definition"""

    type: OpenResponsesResultType
    name: str
    parameters: Nullable[Dict[str, Nullable[Any]]]
    description: NotRequired[Nullable[str]]
    strict: NotRequired[Nullable[bool]]


class OpenResponsesResultToolFunction(BaseModel):
    r"""Function tool definition"""

    type: OpenResponsesResultType

    name: str

    parameters: Nullable[Dict[str, Nullable[Any]]]

    description: OptionalNullable[str] = UNSET

    strict: OptionalNullable[bool] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "strict"]
        nullable_fields = ["description", "strict", "parameters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenResponsesResultToolUnionTypedDict = TypeAliasType(
    "OpenResponsesResultToolUnionTypedDict",
    Union[
        CodexLocalShellToolTypedDict,
        ApplyPatchServerToolTypedDict,
        ShellServerToolTypedDict,
        CodeInterpreterServerToolTypedDict,
        CustomToolTypedDict,
        ComputerUseServerToolTypedDict,
        OpenResponsesResultToolFunctionTypedDict,
        FileSearchServerToolTypedDict,
        LegacyWebSearchServerToolTypedDict,
        WebSearchServerToolTypedDict,
        Preview20250311WebSearchServerToolTypedDict,
        PreviewWebSearchServerToolTypedDict,
        McpServerToolTypedDict,
        ImageGenerationServerToolTypedDict,
    ],
)


OpenResponsesResultToolUnion = Annotated[
    Union[
        Annotated[OpenResponsesResultToolFunction, Tag("function")],
        Annotated[PreviewWebSearchServerTool, Tag("web_search_preview")],
        Annotated[
            Preview20250311WebSearchServerTool, Tag("web_search_preview_2025_03_11")
        ],
        Annotated[LegacyWebSearchServerTool, Tag("web_search")],
        Annotated[WebSearchServerTool, Tag("web_search_2025_08_26")],
        Annotated[FileSearchServerTool, Tag("file_search")],
        Annotated[ComputerUseServerTool, Tag("computer_use_preview")],
        Annotated[CodeInterpreterServerTool, Tag("code_interpreter")],
        Annotated[McpServerTool, Tag("mcp")],
        Annotated[ImageGenerationServerTool, Tag("image_generation")],
        Annotated[CodexLocalShellTool, Tag("local_shell")],
        Annotated[ShellServerTool, Tag("shell")],
        Annotated[ApplyPatchServerTool, Tag("apply_patch")],
        Annotated[CustomTool, Tag("custom")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OpenResponsesResultTypedDict(TypedDict):
    r"""Complete non-streaming response from the Responses API"""

    id: str
    object: OpenResponsesResultObject
    created_at: float
    model: str
    status: OpenAIResponsesResponseStatus
    completed_at: Nullable[float]
    output: List[OutputItemsTypedDict]
    error: Nullable[ResponsesErrorFieldTypedDict]
    r"""Error information returned from the API"""
    incomplete_details: Nullable[IncompleteDetailsTypedDict]
    temperature: Nullable[float]
    top_p: Nullable[float]
    presence_penalty: Nullable[float]
    frequency_penalty: Nullable[float]
    instructions: Nullable[BaseInputsUnionTypedDict]
    metadata: Nullable[Dict[str, str]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""
    tools: List[OpenResponsesResultToolUnionTypedDict]
    tool_choice: OpenAIResponsesToolChoiceUnionTypedDict
    parallel_tool_calls: bool
    user: NotRequired[Nullable[str]]
    output_text: NotRequired[str]
    prompt_cache_key: NotRequired[Nullable[str]]
    safety_identifier: NotRequired[Nullable[str]]
    usage: NotRequired[Nullable[UsageTypedDict]]
    r"""Token usage information for the response"""
    max_tool_calls: NotRequired[Nullable[float]]
    top_logprobs: NotRequired[float]
    max_output_tokens: NotRequired[Nullable[float]]
    prompt: NotRequired[Nullable[StoredPromptTemplateTypedDict]]
    background: NotRequired[Nullable[bool]]
    previous_response_id: NotRequired[Nullable[str]]
    reasoning: NotRequired[Nullable[BaseReasoningConfigTypedDict]]
    service_tier: NotRequired[Nullable[str]]
    store: NotRequired[bool]
    truncation: NotRequired[Nullable[TruncationEnum]]
    text: NotRequired[TextConfigTypedDict]
    r"""Text output configuration including format and verbosity"""


class OpenResponsesResult(BaseModel):
    r"""Complete non-streaming response from the Responses API"""

    id: str

    object: OpenResponsesResultObject

    created_at: float

    model: str

    status: Annotated[
        OpenAIResponsesResponseStatus, PlainValidator(validate_open_enum(False))
    ]

    completed_at: Nullable[float]

    output: List[OutputItems]

    error: Nullable[ResponsesErrorField]
    r"""Error information returned from the API"""

    incomplete_details: Nullable[IncompleteDetails]

    temperature: Nullable[float]

    top_p: Nullable[float]

    presence_penalty: Nullable[float]

    frequency_penalty: Nullable[float]

    instructions: Nullable[BaseInputsUnion]

    metadata: Nullable[Dict[str, str]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""

    tools: List[OpenResponsesResultToolUnion]

    tool_choice: OpenAIResponsesToolChoiceUnion

    parallel_tool_calls: bool

    user: OptionalNullable[str] = UNSET

    output_text: Optional[str] = None

    prompt_cache_key: OptionalNullable[str] = UNSET

    safety_identifier: OptionalNullable[str] = UNSET

    usage: OptionalNullable[Usage] = UNSET
    r"""Token usage information for the response"""

    max_tool_calls: OptionalNullable[float] = UNSET

    top_logprobs: Optional[float] = None

    max_output_tokens: OptionalNullable[float] = UNSET

    prompt: OptionalNullable[StoredPromptTemplate] = UNSET

    background: OptionalNullable[bool] = UNSET

    previous_response_id: OptionalNullable[str] = UNSET

    reasoning: OptionalNullable[BaseReasoningConfig] = UNSET

    service_tier: OptionalNullable[str] = UNSET

    store: Optional[bool] = None

    truncation: Annotated[
        OptionalNullable[TruncationEnum], PlainValidator(validate_open_enum(False))
    ] = UNSET

    text: Optional[TextConfig] = None
    r"""Text output configuration including format and verbosity"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "user",
            "output_text",
            "prompt_cache_key",
            "safety_identifier",
            "usage",
            "max_tool_calls",
            "top_logprobs",
            "max_output_tokens",
            "prompt",
            "background",
            "previous_response_id",
            "reasoning",
            "service_tier",
            "store",
            "truncation",
            "text",
        ]
        nullable_fields = [
            "completed_at",
            "user",
            "prompt_cache_key",
            "safety_identifier",
            "error",
            "incomplete_details",
            "usage",
            "max_tool_calls",
            "max_output_tokens",
            "temperature",
            "top_p",
            "presence_penalty",
            "frequency_penalty",
            "instructions",
            "metadata",
            "prompt",
            "background",
            "previous_response_id",
            "reasoning",
            "service_tier",
            "truncation",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
